# AUTO GENERATED BASED ON Kong 3.4.x, DO NOT EDIT
# Original source path: kong/pdk/enterprise_edition/jwe.lua

from typing import TypeVar, Any, Union, List, Mapping, Tuple, Optional

number = TypeVar('number', int, float)
table = TypeVar('table', List[Any], Mapping[str, Any])
# XXX
cdata = Any
err = str


class jwe():


    @staticmethod
    def decode(token: str) -> Tuple[str, str]:
        """

            This function will return a table that looks like this:
            ```
            {
              [1] = protected header (as it appears in token)
              [2] = encrypted key (as it appears in token)
              [3] = initialization vector (as it appears in token)
              [4] = ciphertext (as it appears in token)
              [5] = authentication tag (as it appears in token)
              protected = protected key (base64url decoded and json decoded)
              encrypted_key = encrypted key (base64url decoded)
              iv = initialization vector (base64url decoded)
              ciphertext = ciphertext (base64url decoded)
              tag = authentication tag (base64url decoded)
              aad = protected header (as it appears in token)
            }
            ```
            The original input can be reconstructed with:
            ```
            local token = table.concat(<decoded-table>, ".")
            ```
            If there is not exactly 5 parts in JWT token, or any decoding fails,
            the error is returned.
            @usage
            local jwe = require "kong.enterprise_edition.jwe"
            local jwt, err = jwe.decode(
              "eyJhbGciOiJFQ0RILUVTIiwiZW5jIjoiQTI1NkdDTSIsImFwdSI6Ik1lUFhUS2oyWFR1NUktYldUSFI2bXci" ..
              "LCJhcHYiOiJmUHFoa2hfNkdjVFd1SG5YWFZBclVnIiwiZXBrIjp7Imt0eSI6IkVDIiwiY3J2IjoiUC0yNTYi" ..
              "LCJ4IjoiWWd3eF9NVXRLTW9NYUpNZXFhSjZjUFV1Z29oYkVVc0I1NndrRlpYRjVMNCIsInkiOiIxaEYzYzlR" ..
              "VEhELVozam1vYUp2THZwTGJqcVNaSW9KNmd4X2YtUzAtZ21RIn19..4ZrIopIhLi3LeXyE.-Ke4ofA.MI5lT" ..
              "kML5NIa-Twm-92F6Q")
            if jwt then
              print(jwt.protected.alg) -- outputs "ECDH-ES"
            end

        :parameter token: JWE encrypted JWT token
        :type token: str

        :return: A table containing JWT token parts decoded, or nil

        :rtype: str
        :return: Error message, or nil

        :rtype: str
        """
        pass

    @staticmethod
    def decrypt(key: Any, token: str) -> Tuple[str, str]:
        """

            Supported keys (`key` argument):
            * Supported key formats:
              * `JWK` (given as a `string` or `table`)
              * `PEM` (given as a `string`)
              * `DER` (given as a `string`)
            * Supported key types:
              * `RSA`
              * `EC`, supported curves:
                * `P-256`
                * `P-384`
                * `P-521`
            @usage
            local jwe = require "kong.enterprise_edition.jwe"
            local jwk = {
              kty = "EC",
              crv = "P-256",
              use = "enc",
              x   = "MKBCTNIcKUSDii11ySs3526iDZ8AiTo7Tu6KPAqv7D4",
              y   = "4Etl6SRW2YiLUrN5vfvVHuhp7x8PxltmWWlbbM4IFyM",
              d   = "870MB6gfuTJ4HtUnUvYMyJpr5eUZNP4Bk43bVdj3eAE",
            }
            local plaintext, err = jwe.decrypt(jwk,
              "eyJhbGciOiJFQ0RILUVTIiwiZW5jIjoiQTI1NkdDTSIsImFwdSI6Ik1lUFhUS2oyWFR1NUktYldUSFI2bXci" ..
              "LCJhcHYiOiJmUHFoa2hfNkdjVFd1SG5YWFZBclVnIiwiZXBrIjp7Imt0eSI6IkVDIiwiY3J2IjoiUC0yNTYi" ..
              "LCJ4IjoiWWd3eF9NVXRLTW9NYUpNZXFhSjZjUFV1Z29oYkVVc0I1NndrRlpYRjVMNCIsInkiOiIxaEYzYzlR" ..
              "VEhELVozam1vYUp2THZwTGJqcVNaSW9KNmd4X2YtUzAtZ21RIn19..4ZrIopIhLi3LeXyE.-Ke4ofA.MI5lT" ..
              "kML5NIa-Twm-92F6Q")
            if plaintext then
              print(plaintext) -- outputs "hello"
            end

        :parameter key: Private key
        :type key: Any
        :parameter token: JWE encrypted JWT token
        :type token: str

        :return: JWT token payload in plaintext, or nil

        :rtype: str
        :return: Error message, or nil

        :rtype: str
        """
        pass

    @staticmethod
    def encrypt(alg: str, enc: str, key: Any, plaintext: str, options: Optional[table]) -> Tuple[str, str]:
        """

            Supported algorithms (`alg` argument):
            * `"RSA-OAEP"`
            * `"ECDH-ES"`
            Supported encryption algorithms (`enc` argument):
            * `"A256GCM"`
            Supported keys (`key` argument):
            * Supported key formats:
              * `JWK` (given as a `string` or `table`)
              * `PEM` (given as a `string`)
              * `DER` (given as a `string`)
            * Supported key types:
              * `RSA`
              * `EC`, supported curves:
                * `P-256`
                * `P-384`
                * `P-521`
            Supported options (`options` argument):
            * `{ zip = "DEF" }`: whether to deflate the plaintext before encrypting
            * `{ apu = <string|boolean> }`: Agreement PartyUInfo header parameter
            * `{ apv = <string|boolean> }`: Agreement PartyVInfo header parameter
            The `apu` and `apv` can also be set to `false` to prevent them from
            being auto-generated (sixteen random bytes) and added to ephemeral
            public key.
            @usage
            local jwe = require "kong.enterprise_edition.jwe"
            local jwk = {
              kty = "EC",
              crv = "P-256",
              use = "enc",
              x   = "MKBCTNIcKUSDii11ySs3526iDZ8AiTo7Tu6KPAqv7D4",
              y   = "4Etl6SRW2YiLUrN5vfvVHuhp7x8PxltmWWlbbM4IFyM",
            }
            local token, err = jwe.encrypt("ECDH-ES", "A256GCM", jwk, "hello", {
              zip = "DEF,
            })
            if token then
              print(token)
            end

        :parameter alg: Algorithm used for key management
        :type alg: str
        :parameter enc: Encryption algorithm used for content encryption
        :type enc: str
        :parameter key: Public key
        :type key: Any
        :parameter plaintext: Plaintext
        :type plaintext: str
        :parameter options: Options (optional), default: nil
        :type options: table

        :return: JWE encrypted JWT token, or nil

        :rtype: str
        :return: Error message, or nil

        :rtype: str
        """
        pass

    pass


# AUTO GENERATED BASED ON Kong 3.4.x, DO NOT EDIT
# Original source path: kong/pdk/enterprise_edition.lua

from typing import TypeVar, Any, Union, List, Mapping, Tuple, Optional

number = TypeVar('number', int, float)
table = TypeVar('table', List[Any], Mapping[str, Any])
# XXX
cdata = Any
err = str

from .jwe import jwe as cls_jwe

class enterprise_edition():

    jwe = cls_jwe

    pass


import os
import re
import sys
import time
import traceback
import threading
import msgpack

from .const import PY3K
if PY3K:
    from socketserver import ThreadingMixIn, UnixStreamServer as sUnixStreamServer
else:
    from SocketServer import ThreadingMixIn, UnixStreamServer as sUnixStreamServer

from gevent import socket as gsocket, sleep as gsleep, spawn as gspawn
from gevent.server import StreamServer as gStreamServer

from .exception import PDKException, PluginServerException

cmdre = re.compile("([a-z])([A-Z])")

DEFAULT_SOCKET_NAME = "python_pluginserver.sock"

def write_response(fd, msgid, response):
    fd.send(msgpack.packb([
        1,  # is response
        msgid,
        None,
        response
    ]))

def write_error(fd, msgid, error):
    fd.send(msgpack.packb([
        1,  # is response
        msgid,
        error,
        None
    ]))


class WrapSocket(object):
    def __init__(self, socket):
        self.socket = socket

    def read(self, n):
        return self.socket.recv(n)

class Server(object):
    def __init__(self, plugin_server):
        self.ps = plugin_server
        self.logger = plugin_server.logger

    def handle(self, fd, address, *_):
        # can't use socket.makefile here, since it returns a blocking IO
        # msgpack.Unpacker only expects read() but no other semantics to exist
        sockf = WrapSocket(fd)
        unpacker = msgpack.Unpacker(sockf, strict_map_key=False)

        for _, msgid, method, args in unpacker:
            ns, cmd = method.split(".")
            if ns != "plugin":
                write_error(fd, msgid, "RPC for %s is not supported" % ns)
                continue

            cmd_r = cmd[0].lower() + cmdre.sub(lambda m: "%s_%s" % (m.group(1), m.group(2).lower()), cmd[1:])
            try:
                self.logger.debug("rpc: #%d method: %s args: %s" % (msgid, method, args))
                ret = getattr(self.ps, cmd_r)(*args)
                self.logger.debug("rpc: #%d return: %s" % (msgid, ret))
                write_response(fd, msgid, ret)
            except (PluginServerException, PDKException) as ex:
                self.logger.warn("rpc: #%d error: %s" % (msgid, str(ex)))
                write_error(fd, msgid, str(ex))
            except Exception as ex:
                self.logger.error("rpc: #%d exception: %s" % (msgid, traceback.format_exc()))
                write_error(fd, msgid, str(ex))

class tUnixStreamServer(ThreadingMixIn, sUnixStreamServer):
    pass

def watchdog(sleep, logger):
    while True:
        if os.getppid() == 1:  # parent dead, process adopted by init
            logger.info("Kong exits, terminating...")
            sys.exit()
        sleep(1)

class UnixStreamServer(Server):
    def __init__(self, pluginserver, path,
                 sock_name=DEFAULT_SOCKET_NAME, use_gevent=True, listen_queue_size=4096):
        Server.__init__(self, pluginserver)
        self.path = os.path.join(path, sock_name)
        self.use_gevent = use_gevent
        self.listen_queue_size = listen_queue_size

    def serve_forever(self):
        if os.path.exists(self.path):
            os.remove(self.path)

        if self.use_gevent:
            listener = gsocket.socket(gsocket.AF_UNIX, gsocket.SOCK_STREAM)
            listener.bind(self.path)
            listener.listen(self.listen_queue_size)

            self.logger.info("server (gevent) started at path " + self.path)

            gspawn(watchdog, gsleep, self.logger)

            gStreamServer(listener, self.handle).serve_forever()
        else:
            import socket
            self.logger.info("server started at path " + self.path)

            t = threading.Thread(
                target=watchdog,
                args=(time.sleep, self.logger, ),
            )
            t.setDaemon(True)
            t.start()
            s = tUnixStreamServer(self.path, self.handle, bind_and_activate=False)
            s.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, True)
            s.server_bind()
            s.socket.listen(self.listen_queue_size)
            s.serve_forever()



# AUTO GENERATED BASED ON Kong 3.4.x, DO NOT EDIT
# Original source path: kong/pdk/nginx.lua

from typing import TypeVar, Any, Union, List, Mapping, Tuple, Optional

number = TypeVar('number', int, float)
table = TypeVar('table', List[Any], Mapping[str, Any])
# XXX
cdata = Any
err = str

from .shared import shared as cls_shared

class nginx():

    shared = cls_shared

    @staticmethod
    def get_ctx(k: str) -> Any:
        """

            get a key-value pair from Kong's per-request context

        :parameter k: key for the ctx data
        :type k: str

        :return: the per-request context data in ngx.ctx

        :rtype: Any
        """
        pass

    @staticmethod
    def get_statistics() -> table:
        """

            Returns various connection and request metrics exposed by
            Nginx, similar to those reported by the
            [ngx_http_stub_status_module](https://nginx.org/en/docs/http/ngx_http_stub_status_module.html#data).
            The following fields are included in the returned table:
            * `connections_active` - the current number of active client connections including `connections_waiting`.
            * `connections_reading` - the current number of connections where nginx is reading the request header.
            * `connections_writing` - the current number of connections where nginx is writing the response back to the client.
            * `connections_waiting` - the current number of idle client connections waiting for a request.
            * `connections_accepted` - the total number of accepted client connections.
            * `connections_handled` - the total number of handled connections. Same as `connections_accepted` unless some resource limits have been reached
              (for example, the [`worker_connections`](https://nginx.org/en/docs/ngx_core_module.html#worker_connections) limit).
            * `total_requests` - the total number of client requests.

        Example:
            nginx_statistics = kong.nginx.get_statistics()

        :return: Nginx connections and requests statistics

        :rtype: table
        """
        pass

    @staticmethod
    def get_subsystem() -> str:
        """

        :return: the subsystem name

        :rtype: str
        """
        pass

    @staticmethod
    def get_tls1_version_str() -> str:
        """

        :return: the TLSv1 version string

        :rtype: str
        """
        pass

    @staticmethod
    def get_var() -> str:
        """

        :return: get NGINX variable value

        :rtype: str
        """
        pass

    @staticmethod
    def req_start_time() -> number:
        """

            get the current request's start timestamp

        :return: req_start_time

        :rtype: number
        """
        pass

    @staticmethod
    def set_ctx(k: str, any: str) -> None:
        """

            set a key-value pair in Kong's per-request context

        :parameter k: key for the ctx data
        :type k: str
        :parameter any: value for the ctx data
        :type any: str

        """
        pass

    pass


# AUTO GENERATED BASED ON Kong 3.4.x, DO NOT EDIT
# Original source path: kong/pdk/plugin.lua

from typing import TypeVar, Any, Union, List, Mapping, Tuple, Optional

number = TypeVar('number', int, float)
table = TypeVar('table', List[Any], Mapping[str, Any])
# XXX
cdata = Any
err = str


class plugin():


    @staticmethod
    def get_id() -> str:
        """

            Returns the instance ID of the plugin.

        Phases:
            rewrite, access, header_filter, response, body_filter, log

        Example:
            kong.request.get_id() # "123e4567-e89b-12d3-a456-426614174000"

        :return: The ID of the running plugin

        :rtype: str
        """
        pass

    pass